#include	<stdlib.h>
#include	<stdio.h>
#include	<cuda.h>

#define	EPS	0.00001f

void randomInit ( float * a, int n )
{
	for ( int i = 0; i < n; i++ )
		a [i] = rand () / (float) RAND_MAX;
}

int main ( int argc, char * argv [] )
{
	const unsigned int blocksSize = 512;
	const unsigned int numBlocks  = 3;
	const unsigned int numItems   = numBlocks * blocksSize;

	CUdevice   hDevice;
	CUcontext  hContext;
	CUmodule   hModule;
	CUfunction hFunction;

							// create CUDA device & context
	cuInit              ( 0 );
	cuDeviceGet         ( &hDevice, 0 ); // pick first device
	cuCtxCreate         ( &hContext, 0, hDevice );
	cuModuleLoad        ( &hModule, "vecAdd.cubin" );
	cuModuleGetFunction ( &hFunction, hModule, "vectorAdd" );

							// allocate host vectors
	float * a = new float [numItems];
	float * b = new float [numItems];
	float * c = new float [numItems];

							// initialize host memory
	randomInit ( a, numItems );
	randomInit ( b, numItems );

							// allocate memory on the device
	CUdeviceptr aDev, bDev, cDev;

	cuMemAlloc ( &aDev, numItems * sizeof ( float ) );
	cuMemAlloc ( &bDev, numItems * sizeof ( float ) );
	cuMemAlloc ( &cDev, numItems * sizeof ( float ) );

							// copy host vectors to device
	cuMemcpyHtoD ( aDev, a, numItems * sizeof ( float ) );
	cuMemcpyHtoD ( bDev, b, numItems * sizeof ( float ) );

							// set up parameter values
	cuFuncSetBlockShape ( hFunction, blocksSize, 1, 1 );

	#define ALIGN_UP(offset, alignment) \
		(offset) = ((offset) + (alignment) - 1) & ~((alignment) - 1)
		
	int    offset = 0;
	void * ptr   = (void*)(size_t)aDev;

	ALIGN_UP( offset, __alignof(ptr));

	cuParamSetv ( hFunction, offset, &ptr, sizeof ( ptr ) );

	offset += sizeof ( ptr );

	ptr = (void*)(size_t)bDev;

	ALIGN_UP(offset, __alignof(ptr));

	cuParamSetv ( hFunction, offset, &ptr, sizeof ( ptr ) );

	offset += sizeof ( ptr );

	ptr = (void*)(size_t)cDev;

	ALIGN_UP(offset, __alignof(ptr));

	cuParamSetv ( hFunction, offset, &ptr, sizeof ( ptr ) );

	offset += sizeof ( ptr );

	cuParamSetSize ( hFunction, offset );

						// execute kernel
	cuLaunchGrid ( hFunction, numBlocks, 1 );

						// copy the result from device back to host
	cuMemcpyDtoH ((void *) c, cDev, numItems * sizeof ( float ) );

	for ( int i = 0; i < numItems; i++ )
		if ( fabs ( a [i] + b [i] - c [i] ) > EPS )
			printf ( "Error at index %d\n", i );
	
	delete[] a;
	delete[] b;
	delete[] c;

	cuMemFree ( aDev );
	cuMemFree ( bDev );
	cuMemFree ( cDev );
}