#include	<stdlib.h>
#include	<stdio.h>
#include	"vecAdd.cu"

#define	EPS	0.00001f

void randomInit ( float * a, int n )
{
	for ( int i = 0; i < n; i++ )
		a [i] = rand () / (float) RAND_MAX;
}

int main ( int argc, char * argv [] )
{
	const unsigned int blockSize = 512;
	const unsigned int numBlocks = 3;
	const unsigned int numItems  = numBlocks * blockSize;

						// create CUDA device & context
	cudaSetDevice( 0 ); // pick first device

						// allocate host vectors
	float * a = new float [numItems];
	float * b = new float [numItems];
	float * c = new float [numItems];

						// initialize host memory
	randomInit ( a, numItems );
	randomInit ( b, numItems );

						// allocate device memory
	float * aDev, * bDev, * cDev;

	cudaMalloc ( (void **)&aDev, numItems * sizeof ( float ) );
	cudaMalloc ( (void **)&bDev, numItems * sizeof ( float ) );
	cudaMalloc ( (void **)&cDev, numItems * sizeof ( float ) );

						// copy host vectors to device
	cudaMemcpy ( aDev, a, numItems * sizeof ( float ), cudaMemcpyHostToDevice );
	cudaMemcpy ( bDev, b, numItems * sizeof ( float ), cudaMemcpyHostToDevice );

						// run kernel
	vectorAdd<<<numBlocks, blockSize>>> ( aDev, bDev, cDev );

						// copy result from device to host
	cudaMemcpy ( (void *) c, cDev, numItems * sizeof ( float ), cudaMemcpyDeviceToHost );

	for ( int i = 0; i < numItems; i++ )
		if ( fabs ( a [i] + b [i] - c [i] ) > EPS )
			printf ( "Error at index %d\n", i );
	
	delete [] a;
	delete [] b;
	delete [] c;

	cudaFree ( aDev );
	cudaFree ( bDev );
	cudaFree ( cDev );
}