#include	<stdio.h>
#include	<stdlib.h>

typedef	unsigned int	uint;

#define	N			(5*1024*1024)
#define BLOCK_SIZE	512

__device__ void Comparator ( uint& keyA, uint& valA, uint& keyB, uint& valB, uint dir )
{
    uint t;
	
    if ( (keyA > keyB) == dir )			// swap (keyA, valA) with (keyB, valB)
	{
        t = keyA; keyA = keyB; keyB = t;
        t = valA; valA = valB; valB = t;
    }
}

__global__ void bitonicSortShared ( uint * dstKey, uint * dstVal, uint * srcKey, uint * ssrcVal, uint arrayLength, uint dir )
{
				// Shared memory storage for one or more short vectors
    __shared__ uint sk [BLOCK_SIZE * 2];
    __shared__ uint sv [BLOCK_SIZE * 2];
	int				index = blockId.x * BLOCK_SIZE * 2 + threadId.x.x;

				// load into shared memory
    sk [threadIdx.x             ] = srcKey [index];
    sv [threadIdx.x             ] = srcVal [index];
    sk [threadIdx.x + BLOCK_SIZE] = srcKey [index + BLOCK_SIZE];
    sv [threadIdx.x + BLOCK_SIZE] = srcVal [index + BLOCK_SIZE];

    for ( uint size = 2; size < arrayLength; size <<= 1 )
	{
								//Bitonic merge
        uint ddd = dir ^ ( (threadIdx.x & (size / 2)) != 0 );
		
        for ( uint stride = size >> 1; stride > 0; stride >>= 1)
		{
            __syncthreads ();
			
            uint pos = 2 * threadIdx.x - (threadIdx.x & (stride - 1));
			
            Comparator ( sk [pos], sv [pos], sk [pos + stride], sv [pos + stride], ddd );
        }
    }

				//last bitonic merge step
    for ( uint stride = arrayLength >> 1; stride > 0; stride >>= 1 )
	{
        __syncthreads ();
			
        uint pos = 2 * threadIdx.x - (threadIdx.x & (stride - 1));
			
        Comparator ( sk [pos], sv [pos], sk [pos + stride], sv [pos + stride], dir );
    }

    __syncthreads ();
	
    dstKey [index             ] = sk [threadIdx.x             ];
    dstVal [index             ] = sv [threadIdx.x             ];
    dstKey [index + BLOCK_SIZE] = sk [threadIdx.x + BLOCK_SIZE];
    dstVal [index + BLOCK_SIZE] = sv [threadIdx.x + BLOCK_SIZE];
}

void randomInit ( int * a, int n )
{
	for ( int i = 0; i < n; i++ )
		a [i] = rand ();
}

int main ( int argc, char * argv [] )
{                
	int				  * array       = new int [N];
	int               * arrayDevIn  = NULL;
	int               * arrayDevOut = NULL;
    cudaEvent_t 		start, stop;
    float 				gpuTime = 0.0f;
	
	randomInit ( array, N );
	
    cudaEventCreate ( &start );
    cudaEventCreate ( &stop );
    cudaEventRecord ( start, 0 );
	
	cudaMalloc ( (void **) &arrayDevIn,  N * sizeof ( int ) );
	cudaMalloc ( (void **) &arrayDevOut, N * sizeof ( int ) );
    cudaMemcpy ( arrayDevIn, array, N * sizeof ( int ), cudaMemcpyHostToDevice );
	
	bitonicSortShared<<<dim3(N/BLOCK_SIZE/2), dim3(BLOCK_SIZE)>>> ( );
	
    cudaMemcpy ( array, arrayDevOut, N * sizeof ( int ), cudaMemcpyDeviceToHost );

	cudaFree   ( arrayDevIn  );
	cudaFree   ( arrayDevOut );
	
    cudaEventRecord      ( stop, 0 );
	cudaEventSynchronize ( stop );
	cudaEventElapsedTime ( &gpuTime, start, stop );

	printf ( "Elapsed time: %.2f\n", gpuTime );
	
	for ( int i = 1; i < N; i++ )
		if ( array [i-1] > array [i] )
			printf ( "Error at pos %d\n", i );
		
	return 0;
}