#ifndef _CU_INT2_H_
#define _CU_INT2_H_

//**********************************************************************************
// int2 operators and functions
//**********************************************************************************

SDH_CALL int2 operator + (const int2 & u, int v) { int2 r = { u.x + v, u.y + v }; return r; }
SDH_CALL int2 operator - (const int2 & u, int v) { int2 r = { u.x - v, u.y - v }; return r; }
SDH_CALL int2 operator * (const int2 & u, int v) { int2 r = { u.x * v, u.y * v }; return r; }
SDH_CALL int2 operator / (const int2 & u, int v) { int2 r = { u.x / v, u.y / v }; return r; }

SDH_CALL int2 operator + (int v, const int2 & u) { int2 r = { v + u.x, v + u.y }; return r; }
SDH_CALL int2 operator - (int v, const int2 & u) { int2 r = { v - u.x, v - u.y }; return r; }
SDH_CALL int2 operator * (int v, const int2 & u) { int2 r = { v * u.x, v * u.y }; return r; }
SDH_CALL int2 operator / (int v, const int2 & u) { int2 r = { v / u.x, v / u.y }; return r; }

SDH_CALL int2 operator + (const int2 & u, const int2 & v) { int2 r = { u.x + v.x, u.y + v.y }; return r; }
SDH_CALL int2 operator - (const int2 & u, const int2 & v) { int2 r = { u.x - v.x, u.y - v.y }; return r; }
SDH_CALL int2 operator * (const int2 & u, const int2 & v) { int2 r = { u.x * v.x, u.y * v.y }; return r; }
SDH_CALL int2 operator / (const int2 & u, const int2 & v) { int2 r = { u.x / v.x, u.y / v.y }; return r; }

SDH_CALL int2   operator - (const int2 & v) { int2 r = { -v.x, -v.y }; return r; }

SDH_CALL int2 & operator += (int2 & u, const int2 & v) { u.x += v.x; u.y += v.y; return u; }
SDH_CALL int2 & operator -= (int2 & u, const int2 & v) { u.x -= v.x; u.y -= v.y; return u; }
SDH_CALL int2 & operator *= (int2 & u, const int2 & v) { u.x *= v.x; u.y *= v.y; return u; }
SDH_CALL int2 & operator /= (int2 & u, const int2 & v) { u.x /= v.x; u.y /= v.y; return u; }

SDH_CALL int2 & operator += (int2 & u, int v) { u.x += v; u.y += v; return u; }
SDH_CALL int2 & operator -= (int2 & u, int v) { u.x -= v; u.y -= v; return u; }
SDH_CALL int2 & operator *= (int2 & u, int v) { u.x *= v; u.y *= v; return u; }
SDH_CALL int2 & operator /= (int2 & u, int v) { u.x /= v; u.y /= v; return u; }

SDH_CALL int2 catmullrom (const int2 & P0, const int2 & P1, const int2 & P2, const int2 & P3, float t)
{
  const float ts = t * t;
  const float tc = t * ts;

  return (P0 * (-tc + 2.0f * ts - t) + P1 * (3.0f * tc - 5.0f * ts + 2.0f) + P2 * (-3.0f * tc + 4.0f * ts + t) + P3 * ( tc - ts)) * 0.5f;
}
SDH_CALL int2 lerp      (const int2 & u, const int2 & v, float t ) { return u + t * (v - u); }
SDH_CALL int  dot       (const int2 & u, const int2 & v) { return (u.x*v.x + u.y*v.y); }
SDH_CALL int2 clamp     (const int2 & u, int a, int b) { int2 r = { clamp(u.x, a, b), clamp(u.y, a, b) }; return r; }

SDH_CALL int2 saturate  (const int2 & u) { return clamp(u, 0.0f, 1.0f); }

SDH_CALL int2 min(const int2 & u, const int2 & v){ int2 r = {min(u.x, v.x), min(u.y, v.y)}; return r; }
SDH_CALL int2 max(const int2 & u, const int2 & v){ int2 r = {max(u.x, v.x), max(u.y, v.y)}; return r; }

SDH_CALL int  sad (const int2 & u, const int2 & v) { return (abs(u.x - v.x) + abs(u.y - v.y)); } // Sum of Absolute Differences for xy components
SDH_CALL int  ssd (const int2 & u, const int2 & v) { return (sqr(u.x - v.x) + sqr(u.y - v.y)); } // Sum of Squared Differences for xy components

SDH_CALL int2 set(int2 & v, int x = 0, int y = 0) 
{ 
  v.x = x; v.y = y; return v; 
}
SDH_CALL int2 set(int2 & v, int1 x, int y = 0.0f) 
{ 
  v.x = x.x; v.y = y; return v; 
}
SDH_CALL int2 i2() { int2 r = {0, 0}; return r; }
SDH_CALL int2 i2(int x) { int2 r = {x, x}; return r;}
SDH_CALL int2 i2(int x, int y) { int2 r = {x, y}; return r;}
SDH_CALL int2 i2(int1 & u, int y) { int2 r = {u.x, y}; return r; }

SDH_CALL float  length (const int2 & u) { return sqrtf(sqr(u.x) + sqr(u.y)); }

#endif