#ifndef _CU_UINT3_H_
#define _CU_UINT3_H_

//**********************************************************************************
// uint3 operators and functions
//**********************************************************************************

SDH_CALL uint3 operator + (const uint3 & u, uint v) { uint3 r = { u.x + v, u.y + v, u.z + v }; return r; }
SDH_CALL uint3 operator - (const uint3 & u, uint v) { uint3 r = { u.x - v, u.y - v, u.z - v }; return r; }
SDH_CALL uint3 operator * (const uint3 & u, uint v) { uint3 r = { u.x * v, u.y * v, u.z * v }; return r; }
SDH_CALL uint3 operator / (const uint3 & u, uint v) { uint3 r = { u.x / v, u.y / v, u.z / v }; return r; }
SDH_CALL uint3 operator % (const uint3 & u, uint v) { uint3 r = { u.x % v, u.y % v, u.z % v }; return r; }

SDH_CALL uint3 operator + (uint v, const uint3 & u) { uint3 r = { v + u.x, v + u.y, v + u.z }; return r; }
SDH_CALL uint3 operator - (uint v, const uint3 & u) { uint3 r = { v - u.x, v - u.y, v - u.z }; return r; }
SDH_CALL uint3 operator * (uint v, const uint3 & u) { uint3 r = { v * u.x, v * u.y, v * u.z }; return r; }
SDH_CALL uint3 operator / (uint v, const uint3 & u) { uint3 r = { v / u.x, v / u.y, v / u.z }; return r; }
SDH_CALL uint3 operator % (uint v, const uint3 & u) { uint3 r = { v % u.x, v % u.y, v % u.z }; return r; }

SDH_CALL uint3 operator + (const uint3 & u, const uint3 & v) { uint3 r = { u.x + v.x, u.y + v.y, u.z + v.z }; return r; }
SDH_CALL uint3 operator - (const uint3 & u, const uint3 & v) { uint3 r = { u.x - v.x, u.y - v.y, u.z - v.z }; return r; }
SDH_CALL uint3 operator * (const uint3 & u, const uint3 & v) { uint3 r = { u.x * v.x, u.y * v.y, u.z * v.z }; return r; }
SDH_CALL uint3 operator / (const uint3 & u, const uint3 & v) { uint3 r = { u.x / v.x, u.y / v.y, u.z / v.z }; return r; }
SDH_CALL uint3 operator % (const uint3 & u, const uint3 & v) { uint3 r = { u.x % v.x, u.y % v.y, u.z % v.z }; return r; }

SDH_CALL uint3 operator - (const uint3 & u) { uint3 r = { -u.x, -u.y, -u.z }; return r; }

SDH_CALL uint3 & operator += (uint3 & u, const uint3 & v) { u.x += v.x; u.y += v.y; u.z += v.z; return u; }
SDH_CALL uint3 & operator -= (uint3 & u, const uint3 & v) { u.x -= v.x; u.y -= v.y; u.z -= v.z; return u; }
SDH_CALL uint3 & operator *= (uint3 & u, const uint3 & v) { u.x *= v.x; u.y *= v.y; u.z *= v.z; return u; }
SDH_CALL uint3 & operator /= (uint3 & u, const uint3 & v) { u.x /= v.x; u.y /= v.y; u.z /= v.z; return u; }
SDH_CALL uint3 & operator %= (uint3 & u, const uint3 & v) { u.x %= v.x; u.y %= v.y; u.z %= v.z; return u; }

SDH_CALL uint3 & operator += (uint3 & u, uint v) { u.x += v; u.y += v; u.z += v; return u; }
SDH_CALL uint3 & operator -= (uint3 & u, uint v) { u.x -= v; u.y -= v; u.z -= v; return u; }
SDH_CALL uint3 & operator *= (uint3 & u, uint v) { u.x *= v; u.y *= v; u.z *= v; return u; }
SDH_CALL uint3 & operator /= (uint3 & u, uint v) { u.x /= v; u.y /= v; u.z /= v; return u; }
SDH_CALL uint3 & operator %= (uint3 & u, uint v) { u.x %= v; u.y %= v; u.z %= v; return u; }

SDH_CALL uint3 catmullrom (const uint3 & P0, const uint3 & P1, const uint3 & P2, const uint3 & P3, float t)
{
  const float ts = t * t;
  const float tc = t * ts;

  return (P0 * (-tc + 2.0f * ts - t) + P1 * (3.0f * tc - 5.0f * ts + 2.0f) + P2 * (-3.0f * tc + 4.0f * ts + t) + P3 * ( tc - ts)) * 0.5f;
}
SDH_CALL uint3 lerp      (const uint3 & u, const uint3 & v, float t ) { return u + t * (v - u); }
SDH_CALL uint  dot       (const uint3 & u, const uint3 & v) { return (u.x*v.x + u.y*v.y + u.z*v.z); }
SDH_CALL uint3 cross     (const uint3 & u, const uint3 & v) { uint3 r = { u.y*v.z-u.z*v.y, u.z*v.x-u.x*v.z, u.x*v.y-u.y*v.x} ; return r; }
SDH_CALL uint3 clamp     (const uint3 & u, uint a, uint b) { uint3 r = { clamp(u.x, a, b), clamp(u.y, a, b), clamp(u.z, a, b) }; return r; }

SDH_CALL float  triple    (const uint3 & a, const uint3 & b, const uint3 & c ) { return dot(a, cross(b, c) ); }
SDH_CALL float  length    (const uint3 & u) { return sqrtf(sqr(u.x) + sqr(u.y) + sqr(u.z)); }
SDH_CALL uint3   normalize (const uint3 & u) { return u / length(u); }

SDH_CALL uint2 xy        (const uint3 & v) { uint2 u = { v.x, v.y }; return u; }

SDH_CALL uint3 min(const uint3 & u, const uint3 & v){ uint3 r = {min(u.x, v.x), min(u.y, v.y), min(u.z, v.z)}; return r; }
SDH_CALL uint3 max(const uint3 & u, const uint3 & v){ uint3 r = {max(u.x, v.x), max(u.y, v.y), max(u.z, v.z)}; return r; }

SDH_CALL uint  sad(const uint3 & u, const uint3 & v) { return (abs(u.x - v.x) + abs(u.y - v.y) + abs(u.z - v.z)); } // Sum of Absolute Differences for xyz components 
SDH_CALL uint  ssd(const uint3 & u, const uint3 & v) { return (sqr(u.x - v.x) + sqr(u.y - v.y) + sqr(u.z - v.z)); } // Sum of Squared  Differences for xyz components 

SDH_CALL uint3 set(uint3 & v, uint x = 0, uint y = 0, uint z = 0) 
{ 
  v.x = x; v.y = y; v.z = z; return v; 
}
SDH_CALL uint3 set(uint3 & v, uint2 xy, uint z = 0) 
{ 
  v.x = xy.x; v.y = xy.y; v.z = z; return v; 
}
SDH_CALL uint3 ui3() { uint3 r = { 0, 0, 0}; return r; }
SDH_CALL uint3 ui3(uint x) { uint3 r = {x, x, x}; return r;}
SDH_CALL uint3 ui3(uint x, uint y, uint z) { uint3 r = {x, y, z}; return r;}
SDH_CALL uint3 ui3(const uint2 & u, uint z) { uint3 r = {u.x, u.y, z}; return r; }

#endif