#ifndef _CU_UINT4_H_
#define _CU_UINT4_H_

//**********************************************************************************
// uint4 operators and functions
//**********************************************************************************

SDH_CALL uint4 operator + (const uint4 & u, uint v) { uint4 r = { u.x + v, u.y + v, u.z + v, u.w + v }; return r; }
SDH_CALL uint4 operator - (const uint4 & u, uint v) { uint4 r = { u.x - v, u.y - v, u.z - v, u.w - v }; return r; }
SDH_CALL uint4 operator * (const uint4 & u, uint v) { uint4 r = { u.x * v, u.y * v, u.z * v, u.w * v }; return r; }
SDH_CALL uint4 operator / (const uint4 & u, uint v) { uint4 r = { u.x / v, u.y / v, u.z / v, u.w / v }; return r; }
SDH_CALL uint4 operator % (const uint4 & u, uint v) { uint4 r = { u.x % v, u.y % v, u.z % v, u.w % v }; return r; }

SDH_CALL uint4 operator + (uint v, const uint4 & u) { uint4 r = { v + u.x, v + u.y, v + u.z, v + u.w }; return r; }
SDH_CALL uint4 operator - (uint v, const uint4 & u) { uint4 r = { v - u.x, v - u.y, v - u.z, v - u.w }; return r; }
SDH_CALL uint4 operator * (uint v, const uint4 & u) { uint4 r = { v * u.x, v * u.y, v * u.z, v * u.w }; return r; }
SDH_CALL uint4 operator / (uint v, const uint4 & u) { uint4 r = { v / u.x, v / u.y, v / u.z, v / u.w }; return r; }
SDH_CALL uint4 operator % (uint v, const uint4 & u) { uint4 r = { v % u.x, v % u.y, v % u.z, v % u.w }; return r; }

SDH_CALL uint4 operator + (const uint4 & u, const uint4 & v) { uint4 r = { u.x + v.x, u.y + v.y, u.z + v.z, u.w + v.w }; return r; }
SDH_CALL uint4 operator - (const uint4 & u, const uint4 & v) { uint4 r = { u.x - v.x, u.y - v.y, u.z - v.z, u.w - v.w }; return r; }
SDH_CALL uint4 operator * (const uint4 & u, const uint4 & v) { uint4 r = { u.x * v.x, u.y * v.y, u.z * v.z, u.w * v.w }; return r; }
SDH_CALL uint4 operator / (const uint4 & u, const uint4 & v) { uint4 r = { u.x / v.x, u.y / v.y, u.z / v.z, u.w / v.w }; return r; }
SDH_CALL uint4 operator % (const uint4 & u, const uint4 & v) { uint4 r = { u.x % v.x, u.y % v.y, u.z % v.z, u.w % v.w }; return r; }

SDH_CALL uint4 & operator += (uint4 & u, const uint4 & v) { u.x += v.x; u.y += v.y; u.z += v.z; u.w += v.w; return u; }
SDH_CALL uint4 & operator -= (uint4 & u, const uint4 & v) { u.x -= v.x; u.y -= v.y; u.z -= v.z; u.w -= v.w; return u; }
SDH_CALL uint4 & operator *= (uint4 & u, const uint4 & v) { u.x *= v.x; u.y *= v.y; u.z *= v.z; u.w *= v.w; return u; }
SDH_CALL uint4 & operator /= (uint4 & u, const uint4 & v) { u.x /= v.x; u.y /= v.y; u.z /= v.z; u.w /= v.w; return u; }
SDH_CALL uint4 & operator %= (uint4 & u, const uint4 & v) { u.x %= v.x; u.y %= v.y; u.z %= v.z; u.w %= v.w; return u; }

SDH_CALL uint4 & operator += (uint4 & u, uint v) { u.x += v; u.y += v; u.z += v; u.w += v; return u; }
SDH_CALL uint4 & operator -= (uint4 & u, uint v) { u.x -= v; u.y -= v; u.z -= v; u.w -= v; return u; }
SDH_CALL uint4 & operator *= (uint4 & u, uint v) { u.x *= v; u.y *= v; u.z *= v; u.w *= v; return u; }
SDH_CALL uint4 & operator /= (uint4 & u, uint v) { u.x /= v; u.y /= v; u.z /= v; u.w /= v; return u; }
SDH_CALL uint4 & operator %= (uint4 & u, uint v) { u.x %= v; u.y %= v; u.z %= v; u.w %= v; return u; }

SDH_CALL uint4   operator - (const uint4 & v) { uint4 r = { -v.x, -v.y, -v.z, -v.w }; return r; }

SDH_CALL uint4 catmullrom (const uint4 & P0, const uint4 & P1, const uint4 & P2, const uint4 & P3, float t)
{
  const float ts = t * t;
  const float tc = t * ts;

  return (P0 * (-tc + 2.0f * ts - t) + P1 * (3.0f * tc - 5.0f * ts + 2.0f) + P2 * (-3.0f * tc + 4.0f * ts + t) + P3 * ( tc - ts)) * 0.5f;
}

SDH_CALL uint4   lerp      (const uint4 & u, const uint4 & v, float t ) { return u + t * (v - u); }
SDH_CALL uint    dot3      (const uint4 & u, const uint4 & v) { return (u.x*v.x + u.y*v.y + u.z*v.z); }
SDH_CALL uint    dot       (const uint4 & u, const uint4 & v) { return (u.x*v.x + u.y*v.y + u.z*v.z + u.w*v.w); }
SDH_CALL uint    dot       (const uint4 & u, const uint3 & v) { return (u.x*v.x + u.y*v.y + u.z*v.z + u.w); }
SDH_CALL uint    dot       (const uint3 & u, const uint4 & v) { return (u.x*v.x + u.y*v.y + u.z*v.z + v.w); }

SDH_CALL uint4 clamp     (const uint4 & u, uint a, uint b) { uint4 r = { clamp(u.x, a, b), clamp(u.y, a, b), clamp(u.z, a, b), clamp(u.w, a, b) }; return r; }

SDH_CALL uint3 xyz       (const uint4 & v) { uint3 u = { v.x, v.y, v.z }; return u; }

SDH_CALL uint3 min3(const uint4 & u, const uint4 & v){ uint3 r = {min(u.x, v.x), min(u.y, v.y), min(u.z, v.z)}; return r; }
SDH_CALL uint3 max3(const uint4 & u, const uint4 & v){ uint3 r = {max(u.x, v.x), max(u.y, v.y), max(u.z, v.z)}; return r; }
SDH_CALL uint4 min(const uint4 & u, const uint4 & v){ uint4 r = {min(u.x, v.x), min(u.y, v.y), min(u.z, v.z), min(u.w, v.w)}; return r; }
SDH_CALL uint4 max(const uint4 & u, const uint4 & v){ uint4 r = {max(u.x, v.x), max(u.y, v.y), max(u.z, v.z), max(u.w, v.w)}; return r; }

SDH_CALL uint  sad3(const uint4 & u, const uint4 & v) { return (abs(u.x - v.x) + abs(u.y - v.y) + abs(u.z - v.z)); } // Sum of Absolute Differences for xyz components only
SDH_CALL uint  ssd3(const uint4 & u, const uint4 & v) { return (sqr(u.x - v.x) + sqr(u.y - v.y) + sqr(u.z - v.z)); } // Sum of Squared  Differences for xyz components only
SDH_CALL uint  sad (const uint4 & u, const uint4 & v) { return (abs(u.x - v.x) + abs(u.y - v.y) + abs(u.z - v.z) + abs(u.w - v.w)); } // Sum of Absolute Differences for xyzw components
SDH_CALL uint  ssd (const uint4 & u, const uint4 & v) { return (sqr(u.x - v.x) + sqr(u.y - v.y) + sqr(u.z - v.z) + sqr(u.w - v.w)); } // Sum of Squared Differences for xyzw components

SDH_CALL uint4 set(uint4 & v, uint x = 0, uint y = 0, uint z = 0, uint w = 0) 
{ 
  v.x = x; v.y = y; v.z = z; v.w = w; return v; 
}
SDH_CALL uint4 set(uint4 & v, uint3 xyz, uint w = 0) 
{ 
  v.x = xyz.x; v.y = xyz.y; v.z = xyz.z; v.w = w; return v; 
}
SDH_CALL uint4 set(uint4 & v, uint2 xy, uint z = 0, uint w = 0) 
{
  v.x = xy.x; v.y = xy.y; v.z = z; v.w = w; return v; 
}

SDH_CALL uint4 ui4() { uint4 r = {0, 0, 0, 0}; return r;}
SDH_CALL uint4 ui4(uint x) { uint4 r = {x, x, x, x}; return r;}
SDH_CALL uint4 ui4(uint x, uint y, uint z, uint w) { uint4 r = {x, y, z, w}; return r;}
SDH_CALL uint4 ui4(const uint3 & u, uint w) { uint4 r = {u.x, u.y, u.z, w}; return r; }

SDH_CALL float  length3(const uint4 & u) { return sqrtf(sqr(u.x) + sqr(u.y) + sqr(u.z)); }
SDH_CALL float  length (const uint4 & u) { return sqrtf(sqr(u.x) + sqr(u.y) + sqr(u.z) + sqr(u.w)); }

SDH_CALL float4 sqrtf   (const uint4 & u) { float4 r = { sqrtf(u.x), sqrtf(u.y), sqrtf(u.z), sqrtf(u.w) }; return r; }

#endif