#include	<stdio.h>

#define BLOCK_SIZE 	256				
#define	N			(16*256*256)		// size of reduced array 

__global__ void reduce5 ( int * inData, int * outData )
{
	__shared__ int data [BLOCK_SIZE];
	int	  tid = threadIdx.x;
	int	  i   = 2 * blockIdx.x * blockDim.x + threadIdx.x;
	
	data [tid] = inData [i] + inData [i+blockDim.x];		// load into shared memeory
	
	__syncthreads ();
	
	for ( int s = blockDim.x / 2; s > 32; s >>= 1 )
	{
		if ( tid < s )
			data [tid] += data [tid + s];
			
		__syncthreads ();
	}
	
	if ( tid < 32 )					// unroll last iterations
    {
        data [tid] += data [tid + 32]; 
        data [tid] += data [tid + 16]; 
        data [tid] += data [tid +  8]; 
        data [tid] += data [tid +  4]; 
        data [tid] += data [tid +  2]; 
        data [tid] += data [tid +  1]; 
    }
	
	if ( tid == 0 )					// write result of block reduction
		outData [blockIdx.x] = data [0];
}

int reduce ( int * data, int n )
{
	int * sums      = NULL;
	int	  numBlocks = n / 512;
	int	  res       = 0;

	cudaMalloc ( (void **) &sums, numBlocks * sizeof ( int ) );

	reduce5<<< dim3 ( numBlocks ), dim3 ( BLOCK_SIZE ) >>> ( data, sums );

			// now we have to reduce sums
	if ( numBlocks > BLOCK_SIZE )
		res = reduce ( sums, numBlocks );
	else
	{
		int * sumsHost = new int [numBlocks];
		
		cudaMemcpy ( sumsHost, sums, numBlocks * sizeof ( int ), cudaMemcpyDeviceToHost );
	
		for ( int i = 0; i < numBlocks; i++ )
			res += sumsHost [i];
			
		delete [] sumsHost;
	}

	cudaFree ( sums );
	
	return res;
}

int main ( int argc, char *  argv [] )
{
	int	numBytes = N * sizeof ( int );
	int	n        = N;
	int	i        = 0;
	int	sum      = 0;

					// allocate host memory
    int * a = new int [N];
	
					// init with random values
	for ( i = 0; i < N; i++ )
	{
		a [i] = 1; //(rand () & 0xFF) - 127;
		sum  += a [i];
	}
		
					// allocate device memory
    int       * adev = NULL;
    cudaEvent_t start, stop;
    float 		gpuTime = 0.0f;
	
    cudaMalloc ( (void**)&adev, numBytes );

					// create cuda event handles
    cudaEventCreate ( &start );
    cudaEventCreate ( &stop );
	
					// asynchronously issue work to the GPU (all to stream 0)
    cudaEventRecord ( start, 0 );
    cudaMemcpy      ( adev, a, numBytes, cudaMemcpyHostToDevice );
		
	int	result = reduce ( adev, N );
	
    cudaEventRecord ( stop, 0 );

	cudaEventSynchronize ( stop );
	cudaEventElapsedTime ( &gpuTime, start, stop );

	printf ( "n= %d GPU result = %d CPU result = %d\n", n, result, sum );
	
						// print the cpu and gpu times
    printf ( "time spent executing by the GPU: %.2f milliseconds\n", gpuTime );

					// release resources
    cudaEventDestroy ( start );
    cudaEventDestroy ( stop  );
    cudaFree         ( adev );

    delete a;

    return 0;
}

