//
// Perform "naive" parallel reduction
//

#include	<stdio.h>
#include	<stdlib.h>

#define BLOCK_SIZE 	256				
#define	N			(16*256*256)		// size of reduced array 

__global__ void reduce5 ( int * inData, int * outData )
{
	__shared__ int data [BLOCK_SIZE];
	int	  tid = threadIdx.x;
	int	  i   = 2 * blockIdx.x * blockDim.x + threadIdx.x;
	
	data [tid] = inData [i] + inData [i+blockDim.x];		// load into shared memeory
	
	__syncthreads ();
	
	for ( int s = blockDim.x / 2; s > 32; s >>= 1 )
	{
		if ( tid < s )
			data [tid] += data [tid + s];
			
		__syncthreads ();
	}
	
	if ( tid < 32 )					// unroll last iterations
    {
        data [tid] += data [tid + 32]; 
        data [tid] += data [tid + 16]; 
        data [tid] += data [tid +  8]; 
        data [tid] += data [tid +  4]; 
        data [tid] += data [tid +  2]; 
        data [tid] += data [tid +  1]; 
    }
	
	if ( tid == 0 )					// write result of block reduction
		outData [blockIdx.x] = data [0];
}

int main ( int argc, char *  argv [] )
{
	int	numBytes = N * sizeof ( int );
	int	n        = N;
	int	i        = 0;
	int	sum      = 0;

					// allocate host memory
    int * a = new int [N];
	int * b = new int [N];
	
					// init with random values
	for ( i = 0; i < N; i++ )
	{
		a [i] = 1; //(rand () & 0xFF) - 127;
		sum  += a [i];
	}
		
					// allocate device memory
    int       * adev [2] = { NULL, NULL };
    cudaEvent_t start, stop;
    float 		gpuTime = 0.0f;
	
    cudaMalloc ( (void**)&adev [0], numBytes );
    cudaMalloc ( (void**)&adev [1], numBytes );

					// create cuda event handles
    cudaEventCreate ( &start );
    cudaEventCreate ( &stop );
	
					// asynchronously issue work to the GPU (all to stream 0)
    cudaEventRecord ( start, 0 );
    cudaMemcpy      ( adev [0], a, numBytes, cudaMemcpyHostToDevice );
	
	for ( i = 0; n >= BLOCK_SIZE; n /= (2*BLOCK_SIZE), i ^= 1 )
	{
					// set kernel launch configuration
		dim3 dimBlock ( BLOCK_SIZE, 1, 1 );
		dim3 dimGrid  ( n / (2*dimBlock.x), 1, 1 );

		printf ( "n= %d i = %d\n", n, i );
		reduce5<<<dimGrid, dimBlock>>> ( adev [i], adev [i^1] );
	}
	
    cudaMemcpy      ( b, adev [i], 4*N, cudaMemcpyDeviceToHost );
    cudaEventRecord ( stop, 0 );

	cudaEventSynchronize ( stop );
	cudaEventElapsedTime ( &gpuTime, start, stop );

	printf ( "n= %d i = %d\n", n, i );
	
	for ( i = 1; i < n; i++ )
		b [0] += b [i];
						// print the cpu and gpu times
    printf ( "time spent executing by the GPU: %.2f milliseconds\n", gpuTime );
	printf ( "CPU sum %d, CUDA sum %d, N = %d\n", sum, b [0], N );

					// release resources
    cudaEventDestroy ( start );
    cudaEventDestroy ( stop  );
    cudaFree         ( adev [0] );
    cudaFree         ( adev [1] );

    delete a;
	delete b;

    return 0;
}
