#include	<cudpp.h>
#include	<stdio.h>
#include	<stdlib.h>

#define	N				(64*256*256)		// size of array 

int main ( int argc, char *  argv [] )
{
	int	numBytes = N * sizeof ( float );
	int	i        = 0;

					// allocate host memory
    float * a = new float [N];
	float * b = new float [N];
	
					// init with random values
	for ( i = 0; i < N; i++ )
		a [i] = 1; //(rand () & 0xFF) - 127;
		
					// allocate device memory
    float     * adev [2] = { NULL, NULL };
    cudaEvent_t start, stop;
    float 		gpuTime = 0.0f;
	
    cudaMalloc ( (void**)&adev [0], numBytes );
    cudaMalloc ( (void**)&adev [1], numBytes );

											// Scan configuration
    CUDPPHandle 		plan;   
    CUDPPConfiguration	config;
	
    config.algorithm = CUDPP_SCAN;
    config.op        = CUDPP_ADD;			// m.b. CUDPP_MAX; CUDPP_MIN, CUDPP_MULTIPLY
    config.datatype  = CUDPP_FLOAT;  		// m.b. CUDPPP_UINT, CUDPP_CHAR, CUDPP_UCHAR, CUDPP_INT, 
    config.options   = CUDPP_OPTION_FORWARD | CUDPP_OPTION_EXCLUSIVE;
	
    cudppPlan ( &plan, config, N, 1, 0 );
    	
					// create cuda event handles
    cudaEventCreate ( &start );
    cudaEventCreate ( &stop );
	
					// asynchronously issue work to the GPU (all to stream 0)
    cudaEventRecord ( start, 0 );
    cudaMemcpy      ( adev [0], a, numBytes, cudaMemcpyHostToDevice );

    cudppScan ( plan, adev [1], adev [0], N );
	
    cudaMemcpy      ( b, adev [1], numBytes, cudaMemcpyDeviceToHost );
    cudaEventRecord ( stop, 0 );

	cudaEventSynchronize ( stop );
	cudaEventElapsedTime ( &gpuTime, start, stop );

	for ( i = 0; i < 3000; i++ )
		if ( fabs ( b [i] - i ) > 0.0001 )
			printf ( "item at %d diff %f --> (%f %d)\n", i, b [i] - i, b [i], i );
		
	printf ( "\n" );
						// print the cpu and gpu times
    printf ( "time spent executing by the GPU: %.2f milliseconds\n", gpuTime );

					// release resources
	cudppDestroyPlan ( plan );
					
    cudaEventDestroy ( start );
    cudaEventDestroy ( stop  );
    cudaFree         ( adev [0] );
    cudaFree         ( adev [1] );

    delete a;
	delete b;

    return 0;
}

