//
// Perform "naive" matrix transpose
//

#include	<stdio.h>
#include	<stdlib.h>

#define BLOCK_SIZE 	16	
#define	N			2048		// matrices will be N*N		

__global__ void transpose1 ( float * inData, float * outData, int n )
{
	unsigned int xIndex   = blockDim.x * blockIdx.x + threadIdx.x;
	unsigned int yIndex   = blockDim.y * blockIdx.y + threadIdx.y;  
	unsigned int inIndex  = xIndex + n * yIndex;
	unsigned int outIndex = yIndex + n * xIndex;
	
	outData [outIndex] = inData [inIndex]; 
}

int main ( int argc, char *  argv [] )
{
	int	numBytes = N * N * sizeof ( float );

					// allocate host memory
    float * a = new float [N*N];
	float * b = new float [N*N];
	
	for ( int i = 0; i < N; i++ )
		for ( int j = 0; j < N; j++ )
		{
			a [i] = 0.0f;
			b [i] = 1.0f;
		}
		
					// allocate device memory
    float * adev = NULL;
    float * bdev = NULL;
	
    cudaMalloc ( (void**)&adev, numBytes );
    cudaMalloc ( (void**)&bdev, numBytes );

					// set kernel launch configuration
	dim3 threads ( BLOCK_SIZE, BLOCK_SIZE );
	dim3 blocks  ( N / threads.x, N / threads.y);

					// create cuda event handles
    cudaEvent_t start, stop;
    float 		gpuTime = 0.0f;

    cudaEventCreate ( &start );
    cudaEventCreate ( &stop );
	
					// asynchronously issue work to the GPU (all to stream 0)
    cudaEventRecord ( start, 0 );
    cudaMemcpy      ( adev, a, numBytes, cudaMemcpyHostToDevice );
	
    transpose1<<<blocks, threads>>> ( adev, bdev, N );
	
    cudaMemcpy      ( b, bdev, numBytes, cudaMemcpyDeviceToHost );
    cudaEventRecord ( stop, 0 );

	cudaEventSynchronize ( stop );
	cudaEventElapsedTime ( &gpuTime, start, stop );

						// print the cpu and gpu times
    printf("time spent executing by the GPU: %.2f millseconds\n", gpuTime );

					// release resources
    cudaEventDestroy ( start );
    cudaEventDestroy ( stop  );
    cudaFree         ( adev  );
    cudaFree         ( bdev  );

    delete a;
	delete b;

    return 0;
}
